import 'dart:typed_data';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart' show rootBundle;
import 'package:image/image.dart' as imglib;
import 'package:provider/provider.dart';

import '../provider/data.dart';
import '../utils/get_images_list.dart';

class Home extends StatelessWidget {
  static const String id = 'home';

  int _linesCount = 3;
  int _columnsCount = 3;

  double _selectorImageSize = 200;
  double _tipImageSize = 100;
  double _tileImageSize = 80;

  Future<void> resetGame(Data myProvider) async {
    myProvider.updateImage = '';
  }

  String getImageAssetName(String imageCode) {
    return 'assets/images/'+imageCode+'.png';
  }

  Future<void> getImagesList(Data myProvider) async {
    GetImagesList getImagesList;
    getImagesList = GetImagesList();
    await getImagesList.init();
    if (getImagesList.images != null) {
      myProvider.updateImages = getImagesList.images;
    }
  }

  Future<void> selectImage(Data myProvider, String imageCode) async {
    myProvider.updateShuffling = true;
    myProvider.updateImage = imageCode;
    myProvider.updateTiles = await splitImageInTiles(myProvider);
    myProvider.updateShuffling = false;
  }

  Container _buildImageSelectorItem(Data myProvider, String image) {
    return Container(
      child: FlatButton(
        child: Container(
          decoration: BoxDecoration(
            borderRadius: BorderRadius.circular(8),
            border: Border.all(
              color: Colors.blue[200],
              width: 8,
            ),
          ),
          margin: EdgeInsets.all(2),
          child: Image(
            image: AssetImage(getImageAssetName(image)),
            width: _selectorImageSize,
            height: _selectorImageSize,
            fit: BoxFit.fill
          ),
        ),
        onPressed: () { selectImage(myProvider, image); },
      ),
    );
  }

  Container _buildImageSelector(Data myProvider) {
    if (myProvider.images.length == 0) {
      getImagesList(myProvider);
    }
    List images = myProvider.images;

    return Container(
      padding: EdgeInsets.all(5),
      child: ListView(
        children: [
          for (var image in images)
            _buildImageSelectorItem(myProvider, image),
        ],
      ),
    );
  }

  Future<List<Image>> splitImageInTiles(Data myProvider) async {
    String imageAsset = getImageAssetName(myProvider.image);
    Uint8List imageData = (await rootBundle.load(imageAsset))
      .buffer
      .asUint8List();
    imglib.Image image = imglib.decodeImage(imageData);

    int x = 0, y = 0;
    int width = (image.width / _linesCount).round();
    int height = (image.height / _columnsCount).round();

    List<imglib.Image> parts = List<imglib.Image>();
    for (int i = 0; i < _linesCount; i++) {
      for (int j = 0; j < _columnsCount; j++) {
        parts.add(imglib.copyCrop(image, x, y, width, height));
        x += width;
      }
      x = 0;
      y += height;
    }

    List<Image> tiles = List<Image>();
    for (var img in parts) {
      tiles.add(Image.memory(imglib.encodeJpg(img)));
    }

    return tiles;
  }

  Container _buildImageTileItem(Image tile) {
    return Container(
      child: Image(
        image: tile.image,
        width: _tileImageSize,
        height: _tileImageSize,
        fit: BoxFit.fill
      )
    );
  }

  Container _buildTilesetWidget(Data myProvider) {
    List tiles = myProvider.tiles;

    tiles.shuffle();

    int tileIndex = 0;

    Table tileset = Table(
      defaultColumnWidth: IntrinsicColumnWidth(),

      border: TableBorder.all(
        color: Colors.black,
        style: BorderStyle.solid,
        width: 2,
      ),
      children: [
        TableRow(children: [
          Column(children: [_buildImageTileItem(tiles[tileIndex++])]),
          Column(children: [_buildImageTileItem(tiles[tileIndex++])]),
          Column(children: [_buildImageTileItem(tiles[tileIndex++])]),
        ]),
        TableRow(children: [
          Column(children: [_buildImageTileItem(tiles[tileIndex++])]),
          Column(children: [_buildImageTileItem(tiles[tileIndex++])]),
          Column(children: [_buildImageTileItem(tiles[tileIndex++])]),
        ]),
        TableRow(children: [
          Column(children: [_buildImageTileItem(tiles[tileIndex++])]),
          Column(children: [_buildImageTileItem(tiles[tileIndex++])]),
          Column(children: [_buildImageTileItem(tiles[tileIndex++])]),
        ]),
      ],
    );

    return Container(
      margin: EdgeInsets.all(4),
      padding: EdgeInsets.all(4),
      decoration: BoxDecoration(
        color: Colors.blue,
        borderRadius: BorderRadius.circular(4),
        border: Border.all(
          color: Colors.green,
          width: 4,
        ),
      ),
      child: tileset,
    );
  }

  Container _buildTipWidget(Data myProvider) {
    return Container(
      margin: EdgeInsets.all(4),
      padding: EdgeInsets.all(4),
      decoration: BoxDecoration(
        color: Colors.blue,
        borderRadius: BorderRadius.circular(4),
        border: Border.all(
          color: Colors.green,
          width: 4,
        ),
      ),
      child: Image(
        image: AssetImage(getImageAssetName(myProvider.image)),
        width: _tipImageSize,
        height: _tipImageSize,
        fit: BoxFit.fill
      ),
    );
  }

  Container _buildGameWidget(Data myProvider) {
    return Container(
      child: Column(
        mainAxisSize: MainAxisSize.min,
        mainAxisAlignment: MainAxisAlignment.center,
        children: [
          _buildTilesetWidget(myProvider),
          SizedBox(height: 20),
          _buildTipWidget(myProvider),
        ],
      ),
    );
  }

  Container _buildShufflingIndicatorWidget() {
    return Container(
      child: Column(
        mainAxisSize: MainAxisSize.min,
        mainAxisAlignment: MainAxisAlignment.center,
        children: [
          Text(
            '⏳',
            style: TextStyle(
              fontSize: 60,
              fontWeight: FontWeight.w600,
              color: Colors.black,
            ),
          ),
        ],
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    Data _myProvider = Provider.of<Data>(context);

    var content;

    if (_myProvider.shuffling) {
      content = _buildShufflingIndicatorWidget();
    } else {
      if (_myProvider.image == '') {
        content = _buildImageSelector(_myProvider);
      } else {
        content = _buildGameWidget(_myProvider);
      }
    }

    return Scaffold(
      appBar: AppBar(
        title: Text('Puzzle'),
        actions: <Widget>[
          _myProvider.image == ''
          ? Text('')
          : IconButton(
            icon: const Icon(Icons.arrow_left_outlined),
            onPressed: () => resetGame(_myProvider),
          ),
        ],
      ),
      body: SafeArea(
        child: Center(
          child: content
        ),
      )
    );
  }
}
